/**
 *  Copyright 2011 Carsten Gräf
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */

package de.twenty11.skysail.common.filters;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * An OrFilter is a filter with FilterType "OR" with only one level of
 * subfilter
 * 
 * The value can be a list of further filters.
 * 
 * @author carsten
 * 
 */
public class OrFilter extends AbstractFilter {

    /**
     * 
     */
    private static final long serialVersionUID = -4265118450543158146L;

    /**
     * Create an AndFilter object from a Map of String to String. Please note
     * that all filter entries will automatically be "AND"ed.
     * 
     * @param valuesMap
     */
    public static Filter newInstance(Map<String, String> valuesMap) {
        if (valuesMap == null || valuesMap.size() == 0) {
            return new OrFilter(FilterType.EMPTY);
        }
        List<Filter> operands = new ArrayList<Filter>();
        for (String key : valuesMap.keySet()) {
            // Create one filter for each key with non-empty value
            String value = valuesMap.get(key);
            if (value != null) {
                Filter filter = new EqualFilter(key, valuesMap.get(key));
                operands.add(filter);
            }
        }
        return new OrFilter(FilterType.OR, null, operands);
    }

    /**
     * 
     * @param type
     */
    // TODO ugly
    public OrFilter(FilterType type) {
        setFilterType(type);
        setAttribute(null);
        setSubfilters(null);
        setValue(null);
    }

    // TODO make private? new Instance?
    public OrFilter(FilterType type, String attribute, List<Filter> subfilters) {
        setFilterType(type);
        setAttribute(attribute);
        setSubfilters(subfilters);
        setValue(null);
    }

    public OrFilter(FilterType type, String attribute, String value) {
        setFilterType(type);
        setAttribute(attribute);
        setSubfilters(null);
        setValue(value);
    }

    @Override
    public String getValue(String attribute) {

        // TODO Auto-generated method stub
        return "";
    }

    @Override
    public boolean match(Map<String, String> properties) {
        for (Filter filter : getSubfilters()) {
            if (filter.match(properties)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("(|");
        for (Filter filter : getSubfilters()) {
            sb.append(filter.toString());
        }
        sb.append(')');
        return sb.toString();
    }

}
